From ed85ac5d09fa892996a06170422430274e050124 Mon Sep 17 00:00:00 2001
From: Sylvain Munaut <tnt@246tNt.com>
Date: Thu, 24 Sep 2009 01:40:55 +0200
Subject: [PATCH 2/5] [gsm_04_08] Fix gsm48_tx_mm_auth_req implementation

It was mainly missing the key_seq field, causing the
command to just be rejected by the ME.

Signed-off-by: Sylvain Munaut <tnt@246tNt.com>
---
 openbsc/include/openbsc/gsm_04_08.h |    7 +++++++
 openbsc/src/gsm_04_08.c             |    8 +++++---
 2 files changed, 12 insertions(+), 3 deletions(-)

diff --git a/openbsc/include/openbsc/gsm_04_08.h b/openbsc/include/openbsc/gsm_04_08.h
index 99d7747..acca100 100644
--- a/openbsc/include/openbsc/gsm_04_08.h
+++ b/openbsc/include/openbsc/gsm_04_08.h
@@ -78,6 +78,13 @@ struct gsm48_loc_area_id {
 	u_int16_t lac;
 } __attribute__ ((packed));
 
+/* Section 9.2.2 */
+struct gsm48_auth_req {
+	u_int8_t key_seq:4,
+	         spare:4;
+	u_int8_t rand[16];
+} __attribute__ ((packed));
+
 /* Section 9.2.15 */
 struct gsm48_loc_upd_req {
 	u_int8_t type:4,
diff --git a/openbsc/src/gsm_04_08.c b/openbsc/src/gsm_04_08.c
index 30c2db3..1cf738e 100644
--- a/openbsc/src/gsm_04_08.c
+++ b/openbsc/src/gsm_04_08.c
@@ -1233,7 +1233,7 @@ int gsm48_tx_mm_auth_req(struct gsm_lchan *lchan, u_int8_t *rand)
 {
 	struct msgb *msg = gsm48_msgb_alloc();
 	struct gsm48_hdr *gh = (struct gsm48_hdr *) msgb_put(msg, sizeof(*gh));
-	u_int8_t *r;
+	struct gsm48_auth_req *ar = (struct gsm48_auth_req *) msgb_put(msg, sizeof(*ar));
 
 	DEBUGP(DMM, "-> AUTH REQ\n");
 
@@ -1241,10 +1241,12 @@ int gsm48_tx_mm_auth_req(struct gsm_lchan *lchan, u_int8_t *rand)
 	gh->proto_discr = GSM48_PDISC_MM;
 	gh->msg_type = GSM48_MT_MM_AUTH_REQ;
 
+	/* Key Sequence: FIXME fixed to 0 */
+	ar->key_seq = 0;
+
 	/* 16 bytes RAND parameters */
-	r = msgb_put(msg, 16);
 	if (rand)
-		memcpy(r, rand, 16);
+		memcpy(ar->rand, rand, 16);
 
 	return gsm48_sendmsg(msg, NULL);
 }
-- 
1.6.4

